#' Rosenbrock banana function
#'
#' Rosenbrock banana function to be used as a scalar function
#' with a multivariate input and a single parameter.
#'
#' @param x  Vector argument (eg control variables)
#' @param theta Scalar parameter
#' 
#' @return A scalar output: use vector \code{\link{apply}} to get a
#' vector output, see Examples.
#'
#' @export
#' @examples
#' x <- rnorm(3)
#' banana(x, theta = 10)
#'
#' ## vector output for fixed control variables
#'
#' Banana <- with(
#'   list(X = lhcube(10, 3)),
#'   function(theta) apply(X, 1, banana, theta = theta)
#' )
#' 
#' Banana(10)
#'
#' ## Here's an output matrix for a design in theta
#'
#' Theta <- 10^seq(from = 1, to = 2, length = 11)
#' Y <- sapply(Theta, Banana)

banana <- function(x, theta) {
    stopifnot(is.vector(x, mode = "numeric"))
    n <- length(x)
    i <- 1L:(n-1)
    sum((1 - x[i])^2 + theta * (x[i+1L]- x[i]^2)^2)
}
