#' Produce a parallel coordinates plot.
#'
#' A parallel coordinates plot is a simple way to viualise
#' multivariate data, using one vertical axis for each variable, and a
#' line across the axes for each case.  The lines can be distinguished
#' by colour or by line type to show subgroups.
#'
#' @param data matrix or data frame
#' @param lower,upper lower and upper bounds for each axis (see Details)
#' @param log Names of variables to have log scales
#' @param col colour for rows of \code{data}
#' @param lty line type for rows of \code{data}
#' @param lwd line width for rows of \code{data}
#' @param plotorder overplotting ordering of lines
#' @param var.label provide lower and upper values for the axes?
#' @param col.axis colour of the axes
#' @param at relative position of the axes
#'
#' @details This is a slightly more elegant version of \code{parcoord}
#' in the \code{MASS} package.  Variables with no variability are
#' dropped with a warning.
#'
#' If \code{lwd} is supplied, then the thicker lines are plotted on
#' top of the thinner ones.
#'
#' Use \code{par("cex.lab")} to change the scaling of the axis labels
#' and the axis values.  It may also be necessary to increase the
#' first component of \code{par("mar")} to show the variable names.
#'
#' Use \code{lower} and \code{upper} to include \code{ylim} values for
#' the axis of each variable.  If not \code{NULL} these must be
#' appropriate to \code{rbind} with \code{data}.  An \code{NA} in
#' \code{lower} or \code{upper} indicates a data-determined limit.
#' See Examples.
#'
#' @keywords graphics
#' @seealso \code{parcoord} in the \code{MASS} package
#' @export
#' @examples
#' op <- par(no.readonly = TRUE)
#' par(mar = c(5, 1, 1, 1), cex.lab = 0.7)
#' parcoord(iris[, 1:4], col = iris$Species, var.label = TRUE)
#'
#' ## example with ylim values (can also include NAs)
#'
#' parcoord(iris[, 1:4], lower = 0, upper = 10, col = iris$Species, var.label = TRUE)
#' legend("topright", legend = levels(iris$Species), col = 1:3, lty = 1, pch = NA)
#'
#' ## with NA's
#'
#' parcoord(iris[, 1:4], lower = c(NA, NA, 0, 0), upper = 10, col = iris$Species, var.label = TRUE)
#' par(op)

parcoord <- function(data, lower = NULL, upper = NULL, log = NULL,
                     col = 1, lty = 1, lwd = 1, plotorder = NULL,
                     var.label = FALSE, col.axis = "darkgrey", at = 1L:p) {

    X <- as.matrix(data); stopifnot(is.numeric(X))
    if (is.null(colnames(X)))
        colnames(X) <- paste0("X", 1L:ncol(X))

    ## re-order rows if required

    col <- rep(col, length.out = nrow(X))
    lty <- rep(lty, length.out = nrow(X))
    lwd <- rep(lwd, length.out = nrow(X))
        
    if (!is.null(plotorder)) {
        oo <- order(rep(plotorder, length.out = nrow(X)))
        X <- X[oo, ]
        col <- col[oo]
        lty <- lty[oo]
        lwd <- lwd[oo]        
    }
    
    ## handle lower and upper limits by introducing dummy rows, lty =
    ## 0 masks them out

    lty <- rep(lty, length.out = nrow(X))

    if (!is.null(lower)) {
        X <- rbind(X, lower)
        lty <- c(lty, 0)
    }

    if (!is.null(upper)) {
        X <- rbind(X, upper)
        lty <- c(lty, 0)
    }

    ## sort out ranges, remove non-variables

    xrange <- apply(X, 2, range, na.rm = TRUE)
    keep <- apply(xrange, 2, diff) > 0
    if (any(!keep)) {
        warning(sprintf("deleted columns: %s", 
                        paste(colnames(X)[!keep], collapse = ", ")))
        X <- X[, keep, drop=FALSE]
        xrange <- xrange[, keep, drop=FALSE]
    }
    p <- ncol(X)
    nms <- colnames(X)

    ## onto [0, 1] after taking logs, if necessary

    yrange <- xrange # preserve for labels

    if (!is.null(log)) {
        mm <- match(log, nms)
        if (any(is.na(mm)))
            stop(sprintf("unmatched \'log\' values: %s",
                         paste(nms[mm], collapse = ", ")))
        X[, mm] <- log(X[, mm])
        xrange[, mm] <- log(xrange[, mm])
        nms[mm] <- paste(nms[mm], "(log)")
    }

    Y <- (t(X) - xrange[1, ]) / (xrange[2, ] - xrange[1, ]) # onto [0, 1]

    ## do the plot, overlaying the lines

    plot.new()
    cex <- par("cex.lab")
    mgp <- par("mgp")
    if (var.label) mgp[2] <- mgp[2] + 1
    xlim <- range(at)
    plot.window(xlim = xlim, ylim = c(0, 1))
    axis(1, at = at, labels = nms, pos = 0, las = 2, tick = FALSE,
         cex.axis = cex, mgp = mgp)
    for (j in at)
        lines(c(j, j), c(0, 1), col = col.axis)
    matplot(at, Y, type = "l", col = col, lty = lty, lwd = lwd, add = TRUE)

    ## add values

    if (var.label) {
        text(at, 0, formatC(yrange[1, ], digits = 3), pos = 1,
             cex = cex, xpd = NA)
        text(at, 1, formatC(yrange[2, ], digits = 3), pos = 3,
             cex = cex, xpd = NA)
    }

    invisible()
}
